/*======================================================*/
/*														*/
/*	straightFileEx.cpp									*/
/*						(c) HAL 2009-					*/
/*														*/
/*======================================================*/
#include "straightFileEx.h"

straightFileEx::straightFileEx()
{
	this->aperiodicityNumFrequencyLength = -1;
	this->spectrumNumFrequencyLength = -1;
	this->f0 = 0.0;
	this->numFrames = -1;
	this->source = NULL;
	this->specgram = NULL;
}

straightFileEx::~straightFileEx()
{
	if( source ) straightSourceDestroy( source );
	if( specgram ) straightSpecgramDestroy( specgram );
}

VCNT_RESULT	straightFileEx::getAperiodicity( long frameIndex, double *aperiodicityBuffer, long bufferLength )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;
	stBool temp;

	if( source && specgram && 0 <= frameIndex && frameIndex < numFrames ){
		temp = straightSourceGetAperiodicity( straight, source, frameIndex, aperiodicityBuffer, bufferLength );
		if( temp == ST_TRUE )
			result = VCNT_TRUE;
	}

	return result;
}

VCNT_RESULT	straightFileEx::getAperiodicity( long frameIndex, vector<double>& aperiodicityBuffer )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( source && specgram && 0 <= frameIndex && frameIndex < numFrames && 0 < aperiodicityNumFrequencyLength ){
		double* buffer = straightSourceGetAperiodicityPointer( straight,source, frameIndex );

		if( buffer != NULL ){
			aperiodicityBuffer.clear();
			aperiodicityBuffer.resize( aperiodicityNumFrequencyLength );
			for( unsigned long i = 0; i < aperiodicityBuffer.size(); i++ )
				aperiodicityBuffer[i] = buffer[i];

			result = VCNT_TRUE;
		} else {
#ifdef DEBUG
			this->outputError( "straightFileEx: Cannot get aperiodicity pointer." );
#endif
			result = VCNT_OUT_OF_MEMORY;
		}
	}

	return result;
}

double straightFileEx::getF0( long frameIndex )
{
	double result = 0.0;

	if( source && 0 <= frameIndex && frameIndex < numFrames )
		straightSourceGetF0( straight, source, frameIndex, &result );

	return result;
}

VCNT_RESULT straightFileEx::getSpectrum( long frameIndex, double *spectrumBuffer, long bufferLength )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;
	stBool temp;

	if( source && specgram && 0 <= frameIndex && frameIndex < numFrames ){
		temp = straightSpecgramGetSpectrum( straight, specgram, frameIndex, spectrumBuffer, bufferLength );
		if( temp == ST_TRUE )
			result = VCNT_TRUE;
	}

	return result;
}

VCNT_RESULT straightFileEx::getSpectrum( long frameIndex, vector<double>& spectrumBuffer )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( source && specgram && 0 <= frameIndex && frameIndex < numFrames && 0 < spectrumNumFrequencyLength ){
		double* buffer = straightSpecgramGetSpectrumPointer( straight, specgram, frameIndex );

		if( buffer != NULL ){
			spectrumBuffer.clear();
			spectrumBuffer.resize( spectrumNumFrequencyLength );
			for( unsigned long i = 0; i < spectrumBuffer.size(); i++ )
				spectrumBuffer[i] = buffer[i];
			result = VCNT_TRUE;
		} else {
#ifdef DEBUG
			this->outputError( "straightFileEx: Cannot get spectrum pointer." );
#endif
			result = VCNT_OUT_OF_MEMORY;
		}
	}

	return result;
}

VCNT_RESULT straightFileEx::setAperiodicity( long frameIndex, double* aperiodicityBuffer, long bufferLength )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;
	
	if( source && specgram && 0 <= frameIndex && frameIndex < numFrames )
		if( ST_TRUE == straightSourceSetAperiodicity( straight, source, frameIndex, aperiodicityBuffer, bufferLength ) )
			result = VCNT_TRUE;

	return result;
}

VCNT_RESULT straightFileEx::setAperiodicity( long frameIndex, vector<double>& aperiodicityBuffer )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( !( aperiodicityBuffer.empty() ) ){
		double* buffer = new double[aperiodicityBuffer.size()];
		if( buffer != NULL ){
			for( unsigned long i = 0; i < aperiodicityBuffer.size(); i ++ )
				buffer[i] = aperiodicityBuffer[i];
			result = setAperiodicity( frameIndex, buffer, aperiodicityBuffer.size() );
			delete[] buffer;
		} else {
#ifdef DEBUG
			this->outputError( "straightFileEx: Cannot make aperiodicity buffer! Maybe out of memory." );
#endif
			result = VCNT_OUT_OF_MEMORY;
		}
	}

	return result;
}

VCNT_RESULT straightFileEx::setSpectrum( long frameIndex, double* spectrumBuffer, long bufferLength )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;
	
	if( source && specgram && 0 <= frameIndex && frameIndex < numFrames )
		if( ST_TRUE == straightSpecgramSetSpectrum( straight, specgram, frameIndex, spectrumBuffer, bufferLength ) )
			result = VCNT_TRUE;

	return result;
}

VCNT_RESULT straightFileEx::setSpectrum( long frameIndex, vector<double>& spectrumBuffer )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( !( spectrumBuffer.empty() ) ){
		double* buffer = new double[spectrumBuffer.size()];
		if( buffer != NULL ){
			for( unsigned long i = 0; i < spectrumBuffer.size(); i ++ )
				buffer[i] = spectrumBuffer[i];
			result = setSpectrum( frameIndex, buffer, spectrumBuffer.size() );
			delete[] buffer;
		} else {
#ifdef DEBUG
			this->outputError( "straightFileEx: Cannot make spectrum buffer! Maybe out of memory." );
#endif
			result = VCNT_OUT_OF_MEMORY;
		}
	}

	return result;
}

VCNT_RESULT straightFileEx::setF0( long frameIndex, double srcF0 )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( 0 <= frameIndex && frameIndex < numFrames )
		if( straightSourceSetF0( straight, source, frameIndex, srcF0 ) == ST_TRUE )
			result = VCNT_TRUE;

	return result;
}

VCNT_RESULT straightFileEx::setNumFrames( long srcNumFrames )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( 0 < srcNumFrames ){
		if( source ) straightSourceDestroy( source );
		if( specgram ) straightSpecgramDestroy( specgram );

		source = straightSourceInitialize( straight, NULL );
		specgram = straightSpecgramInitialize( straight, NULL );

		if( source && specgram ){
			bool temp = true;
			temp &= ( straightSourceCreateF0( straight, source, srcNumFrames ) == ST_TRUE );
			temp &= ( straightSourceCreateAperiodicity( straight, source, srcNumFrames ) == ST_TRUE );
			temp &= ( straightSpecgramCreate( straight, specgram, srcNumFrames ) == ST_TRUE );

			if( temp ){
				result = VCNT_TRUE;
				setParameters();
			}else{
#ifdef DEBUG
				outputError("Cannot create source or specgram. May be out of memory. " );
#endif
				result = VCNT_OUT_OF_MEMORY;
			}
		} else {
			result = VCNT_UNKNOWN;
		}
	}

	return result;
}

VCNT_RESULT straightFileEx::readStfFile( string fileName )
{
	VCNT_RESULT result = VCNT_FILE_NOT_FOUND;
	StraightFile sf;
	char inputFileName[4096];

#ifdef __GNUC__
	strcpy( inputFileName, fileName.c_str() );
#else
	strcpy_s( inputFileName, 4096, fileName.c_str() );
#endif

	if ((sf = straightFileOpen(inputFileName, "r")) != NULL) {
		if (straightFileLoadHeader(sf, straight) == ST_TRUE) {
			char chunkId[5];
			long chunkSize;

			if( source == NULL )
				source = straightSourceInitialize( straight, NULL );
			if( specgram == NULL )
				specgram = straightSpecgramInitialize( straight, NULL );

			if( source && specgram ){
				while (straightFileLoadChunkInfo(sf, chunkId, &chunkSize) == ST_TRUE) {
					if (strcmp(chunkId, "F0  ") == 0) {
						straightFileLoadF0(sf, source);
					} else if (strcmp(chunkId, "AP  ") == 0) {
						straightFileLoadAperiodicity(sf, source);
					} else if (strcmp(chunkId, "SPEC") == 0) {
						straightFileLoadSpecgram(sf, specgram);
					} else {
						straightFileSkipChunk(sf, chunkSize);
					}
				}
				straightFileClose(sf);
				result = VCNT_TRUE;

				setParameters();
			}
		}
	}

	return result;
}

VCNT_RESULT straightFileEx::writeStfFile( string fileName )
{
	VCNT_RESULT result = VCNT_FALSE;
	StraightFile sf;
	char outputFileName[4096];
#ifdef __GNUC__
	strcpy( outputFileName, fileName.c_str() );
#else
	strcpy_s( outputFileName, 4096, fileName.c_str() );
#endif

	if( source && specgram ){
		if ((sf = straightFileOpen(outputFileName, "w")) != NULL) {

			straightFileSetChunkList(sf, "F0  AP  SPEC");
			straightFileWriteHeader(sf, straight);
			straightFileWriteF0Chunk(sf, source);
			straightFileWriteAperiodicityChunk(sf, source);
			straightFileWriteSpecgramChunk(sf, specgram);

			straightFileClose(sf);

			result = VCNT_TRUE;
		}
	} else {
		result = VCNT_INVALID_CALL;
	}
	return result;
}

VCNT_RESULT straightFileEx::straightCompute( waveFileEx &wave )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;
	unsigned long length;

	if( ( length = wave.getWaveLength() ) != 0 ){

		if( source == NULL )
			source = straightSourceInitialize( straight, NULL );
		if( specgram == NULL )
			specgram = straightSpecgramInitialize( straight,NULL );

		double* inputWave = NULL;
		inputWave = new double[length];

		if( inputWave ){
			wave.getWaveBuffer( inputWave, &length );
			straightSourceCompute( straight, source, inputWave, length );
			straightSpecgramCompute( straight, source, specgram, inputWave, length );
			delete[] inputWave;

			result = VCNT_TRUE;
			
			setParameters();
		}else{
#ifdef DEBUG
			outputError( "straightFileEx:Cannot create wave buffer. Maybe out of memory." );
#endif
			result = VCNT_OUT_OF_MEMORY;
		}
	}
	return result;
}

VCNT_RESULT straightFileEx::straightSynthesis( waveFileEx &wave )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;
	
	if( source && specgram ){
		StraightSynth synth = straightSynthInitialize( straight, NULL );
		double* temp;
		long length;

		if( synth ){
			if( straightSynthCompute( straight, source, specgram, synth, 1.0, 1.0, 1.0 ) == ST_TRUE ){
				temp = straightSynthGetOutputWave( synth, &length );
				wave.setWaveBuffer( temp, length );
				result = VCNT_TRUE;
			}
		}else{
#ifdef DEBUG
			outputError( "straightFileEx: Failed initializing straightSynthesis." );
#endif
				result = VCNT_FALSE;
		}
	}
	return result;
}

void	straightFileEx::setParameters()
{
	numFrames = straightSpecgramGetNumFrames( specgram );
	aperiodicityNumFrequencyLength = straightSourceGetAperiodicityFrequencyLength( source );
	spectrumNumFrequencyLength = straightSpecgramGetFrequencyLength( specgram );
}

VCNT_RESULT	straightFileEx::setFrame( long frameIndex, straightFrameEx& frame )
{
	VCNT_RESULT	result = VCNT_FALSE;

	bool temp = true;
	vector<double> apBuffer,spBuffer;
	double tempF0;

	temp &= ( frame.setBufferLength( aperiodicityNumFrequencyLength, spectrumNumFrequencyLength ) == VCNT_TRUE );
	temp &= ( frame.getAperiodicity( apBuffer ) == VCNT_TRUE );
	temp &= ( frame.getSpectrum( spBuffer ) == VCNT_TRUE );
	tempF0 = frame.getF0();

	if( temp ){
		setF0( frameIndex, tempF0 );
		result = setAperiodicity( frameIndex, apBuffer );
		if( result == VCNT_TRUE )
			result = setSpectrum( frameIndex, spBuffer );
	}
	return result;
}

VCNT_RESULT	straightFileEx::getFrame( long frameIndex, straightFrameEx& frame )
{
	VCNT_RESULT result = VCNT_FALSE;

	bool temp = true;
	vector<double> apBuffer,spBuffer;
	double tempF0;

	temp &= ( getAperiodicity( frameIndex, apBuffer ) == VCNT_TRUE );
	temp &= ( getSpectrum( frameIndex, spBuffer ) == VCNT_TRUE );
	tempF0 = getF0( frameIndex );

	if( temp ){
		frame.setF0( tempF0 );
		result = frame.setAperiodicity( apBuffer );
		if( result == VCNT_TRUE )
			result = frame.setSpectrum( spBuffer );
	}
	return result;
}

